//********************************************************************
//*** ResourcePool/IgnoreExceptionCommand.t.cc
//*** Copyright (c) 2003-2008 by Markus Winand <mws@fatalmind.com>
//*** $Id: IgnoreExceptionCommand.t.cc,v 1.2 2008/01/27 16:42:27 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */


#include "t/main.hh"
#include "util/Clone.hh"
#include "IgnoreExceptionCommand.hh"
//#include "BatchCommand.cc"

using namespace fatalmind;

// same as in Execute.t.cc
#define THIS_IS_A_VERY_STRANGE_INCLUDE
#include "ResourcePool.t.cc"

typedef  Command<myType<DefaultThreadedModel> > cmdtype;
class MyTestCommandOK: public cmdtype {
	public:
		MyTestCommandOK(const int id)
		: _called(0)
		, _id(id)
		{
		};
		void execute(Rec& res) {
			_called++;
		}
		int _called;
		int _id;
	protected:
		virtual Clone* DoClone() const {
			return new MyTestCommandOK(*this);
		}
		MyTestCommandOK(const MyTestCommandOK& rho) 
		: _called(rho._called)
		, _id(rho._id)
		{
		};
	private:
		MyTestCommandOK();
};

class MyTestCommandEx: public MyTestCommandOK {
	public:
		MyTestCommandEx(int i = 0, bool nfe = false) 
		: MyTestCommandOK(i)
		, nfe_(nfe)
		{
		}
		void execute(Rec& res) {
			MyTestCommandOK::execute(res);
			if (nfe_) {
				throw NoFailoverException("hirsch");
			} else {
				throw Exception("hirsch");
			}
		}
	protected:
		virtual Clone* DoClone() const {
			return new MyTestCommandEx(*this);
		}
		bool nfe_;
		MyTestCommandEx(const MyTestCommandEx& rho) 
		: MyTestCommandOK(rho)
		, nfe_(rho.nfe_)
		{
		}
};

typedef IgnoreExceptionCommand<MyTestCommandOK> cmd_t;

void TEST_IgnoreExceptionCommand_one() {
	MyTestCommandOK	cmdok(0);
	cmd_t bc(cmdok);

	Fac F("hirsch");	
	RP<> p(F);
	p.execute(cmdok);
	assertEqual(cmdok._called, 1);

	p.execute(bc);
	assertEqual(bc.getCommandRef()._called, 1); // since it was copied prior the first execution
}

void TEST_IgnoreExceptionCommand_FAIL() {
	MyTestCommandEx	cmdex(1);
	cmd_t bc(cmdex);

	Fac F("hirsch");
	RP<> p(F);

	p.execute(bc);
}
