//********************************************************************
//*** ResourcePool/SQL/ConcreteWrapper.hh
//*** Copyright (c) 2003-2008 by Markus Winand <mws@fatalmind.com>
//*** $Id: ConcreteWrapper.hh,v 1.4 2008/01/27 16:42:28 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#include "mm/gc_ptr.hh"
#include "ResourcePool/SQL/Resource.hh"
#include "ResourcePool/SQL/RTSQLCommand.hh"

namespace fatalmind {
namespace SQL {

/*
 * This wrapper can be used to encapsulate a RTSQL Command
 * so that it can be executed by a ResourcePool for a concrete 
 * database.
 * This is useful where commands itself expect child commands as
 * parameter (e.g. Transactions) so that i can put a RTSQL command
 * wrapped with this class into the transaction.
 */
// CPT - Concrete Pool Type (e.g. oracle, mysql, ...)
// RTPT - Runtime Selectable Pool Type
template<class CPT, class RTPT>
class ConcreteWrapper: public Command<CPT>
{
	public:
		ConcreteWrapper(const Command<RTPT>& cmd) 
		: cmd_(cmd.template clone<Command<RTPT> >())
		{
		}

		virtual void execute(typename CPT::resource_t& r) {
			SQLResourceSpecialized<typename RTPT::ThreadingModell, typename CPT::resource_t> rtresource(r);
			cmd_->execute(rtresource);
		}

	private:
		virtual Clone* DoClone() const {
			return new ConcreteWrapper<CPT, RTPT>(*this);
		}

	private:
		gc_ptr<Command<RTPT> > cmd_;
};

}; // namespace SQL
}; // namespace fatalmind
