//********************************************************************
//*** ResourcePool/SQL/RTSQLBatch.t.cc
//*** Copyright (c) 2003-2008 by Markus Winand <mws@fatalmind.com>
//*** $Id: RTSQLBatch.t.cc,v 1.4.2.2 2008/04/13 11:56:51 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#include <memory>
#include "t/main.hh"
#include "config.h"
#include "ResourcePool/ResourcePool.hh"
#include "ResourcePool/SQL/ResourcePool.hh"

#include "testconfig.cc"

#include "SQLBatch.t.cc"

#ifdef HAVE_ORACLE
std::auto_ptr<RP> oci9_rp;
#endif
#ifdef HAVE_MYSQL
std::auto_ptr<RP> mysql3_rp;
#endif

void INIT_SQL_SQLSelect() {
#ifdef HAVE_ORACLE
	oci9_rp.reset(new RP(getFactory("oci:" + getOracleLogon())));
	test_SQLBatch_init(*oci9_rp);
#endif
#ifdef HAVE_MYSQL
	mysql3_rp.reset(new RP(getFactory("mysql:" + getMysqlLogon())));
	test_SQLBatch_init(*mysql3_rp);
#endif
}


void TEST_SQL_OCI9_SQLBatch_one() {
#ifdef HAVE_ORACLE
	test_SQLBatch_one(*oci9_rp);
#endif
}

void TEST_SQL_OCI9_SQLBatch_fail() {
#ifdef HAVE_ORACLE
	test_SQLBatch_fail(*oci9_rp);
#endif
}

void TEST_PERF_SQL_OCI9_SQLBatch_ok() {
#ifdef HAVE_ORACLE
	test_PERF_SQLBatch_ok(*oci9_rp);
#endif
}


void TEST_SQL_MYSQL3_SQLBatch_one() {
#ifdef HAVE_MYSQL
	test_SQLBatch_one(*mysql3_rp);
#endif
}

void TEST_SQL_MYSQL3_SQLBatch_fail() {
#ifdef HAVE_MYSQL
	test_SQLBatch_fail(*mysql3_rp);
#endif
}

void TEST_PERF_SQL_MYSQL3_SQLBatch_ok() {
#ifdef HAVE_MYSQL
	test_PERF_SQLBatch_ok(*mysql3_rp);
#endif
}
