//********************************************************************
//*** ResourcePool/SQL/SQLSelectRow.t.cc
//*** Copyright (c) 2003-2008 by Markus Winand <mws@fatalmind.com>
//*** $Id: SQLSelectRow.t.cc,v 1.16.2.1 2008/04/13 11:56:51 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#include <memory>
#include "t/main.hh"

using namespace std;
using namespace fatalmind;

#include <memory>
#include <iostream>

void test_SQLSelectRow_init(RP& p)
{
	init(p);

	RP::Command::SQLExecute init(p, "insert into hirsch (id, ukn) values (?, ?)");
	init.bind<std::string>(1, "hohoho ' kasdfjs \" sdf");
	for (int i = 0; i < 600; ++i) {
		init.bind(0, i);
		p.execute(init);
	}
}


void test_SQLSelectRow_one(RP& p)
{
	RP::Command::SQLSelectRow cmd(p, "select ukn,id  from hirsch where id = ?");
	int col1;
	std::string col2;

	cmd.bind(0, 4);
	cmd.bindout(0, col2);
	cmd.bindout(1, col1);

	p.execute(cmd);

	assertEqualT(std::string, col2, "hohoho ' kasdfjs \" sdf");
	assertEqual(col1, 4);
}

void test_perf_SQLSelectRow_one_spec_outer(RP& p)
{
	RP::Command::SQLSelectRow cmd(p, "select id, ukn from hirsch where id = ?");
	int col1;
	std::string col2;

	cmd.bindout(0, col1);
	cmd.bindout(1, col2);
	for (int i = 0; i < 600; ++i) {
		cmd.bind(0, i);
		p.execute(cmd);

		assertEqualT(std::string, col2, "hohoho ' kasdfjs \" sdf");
		assertEqual(col1, i);
		assertNotGrowing(i);
	}
}

void test_perf_SQLSelectRow_one_spec_inner(RP& p)
{
	int col1;
	std::string col2;

	for (int i = 0; i < 600; ++i) {
		RP::Command::SQLSelectRow cmd(p, "select id, ukn from hirsch where id = ?");
		cmd.bindout(0, col1);
		cmd.bindout(1, col2);

		cmd.bind(0, i);
		p.execute(cmd);

		assertEqualT(std::string, col2, "hohoho ' kasdfjs \" sdf");
		assertEqual(col1, i);
		assertNotGrowing(i-1);
	}
}

// TODO: mysql?, others
void test_perf_SQLSelectRow_first_one(RP& p)
{
	RP::Command::SQLSelectRow cmd(p, "select id, ukn from hirsch where rownum <= 1");
	int col1;
	std::string col2;

	cmd.bindout(0, col1);
	cmd.bindout(1, col2);
	for (int i = 0; i < 3000; ++i) {
		p.execute(cmd);

		assertEqualT(std::string, col2, "hohoho ' kasdfjs \" sdf");
//		assertNotGrowing(i-100);
		fatalmind_trace_malloc = 1;
	}
	dumpLeakedMemory();
	fatalmind_trace_malloc = 0;
}
// TODO: check behaviour if not found or more then one found..

void test_SQLSelectRow_clone(RP& p)
{
	RP::Command::SQLSelectRow cmd(p, "select id, ukn from hirsch where id = ?");
	int col1;
	std::string col2;
	std::string col22;

	cmd.bindout(0, col1);

	std::auto_ptr<RP::Command::SQLSelectRow> cmd2(cmd.clone<RP::Command::SQLSelectRow>());
	cmd.bindout(1, col2);
	cmd.bind(0, 45);
	cmd2->bindout(1, col22);
	cmd2->bind(0, 46);
	p.execute(cmd);
	assertEqual(col1, 45);
	assertEqual(col22, ""); // unchanged
	p.execute(*cmd2);
	assertEqual(col1, 46);
	assertNotEqualT(std::string, col22, ""); // unchanged
	assertEqual(col2, col22);
}

void test_SQLSelectRow_cleanup(RP& p) 
{
    drop(p);
}
